class CategoriesController < ApplicationController
  include PCT::ResourceController
  
  before_filter :find_category, :except => [ :index, :new, :create ]
  
  # GET /categories
  # TODO: Include count of paths in each category in XML
  def index
    find_categories

    respond_to do |format|
      format.xml { render :xml => (@categories.blank? ? empty_xml_document('categories') : @categories.to_xml(:only => Category.xml_fields)) }
      format.html
    end
  end
  
  def new
    # Only admin users can create new categories
    with_authorization do
      @category = Category.new
    end
  end
  
  # POST /categories
  def create
    @category = Category.new
    update
  end
  
  # GET /categories/{id}
  def show
    respond_to do |format|
      format.xml { render :xml => @category.to_xml(:only => XML_ATTRS) }
      format.html { find_paths }
    end
  end
  
  def update
    # Only admin users can create & update categories
    with_authorization do
      if params[:category]
        @category.attributes = params[:category]
      end
    
      save_resource( @category, [ @category ], :xml_only => Category.xml_fields )
    end
  end
  
  def destroy
    # Only authenticated user can delete their own paths
    with_authorization do
      respond_to do |format|
        format.xml do
          @category.destroy ? http_ok : http_internal_server_error
        end
        format.html do
          if @category.destroy
            flash[:notice] = "Category deleted."
            redirect_to categories_url
          else
            flash[:warning] = "Failed to delete category."
            render :template => 'delete'
          end
        end
      end
    end
  end
  
  # GET /categories/{id}/paths
  def paths
    find_paths
    respond_to do |format|
      format.xml { render_paths_as_xml @paths }
      format.html
    end
  end
  
  protected
  def find_category
    unless @category = Category.find(params[:id])
      logger.debug("Category not found: id #{params[:id]}")
      http_not_found
    end
  end
  
  def find_paths
    @paths = @category.paths.paginate(:page => params[:page], :order => 'name ASC')
  end
end
