class LoginController < ApplicationController
  filter_parameter_logging :password
  
  # Session-based login
  def login
    if request.post? && params[:user]
      if @user = User.authenticate(params[:user][:name], params[:user][:password])
        @authenticated_user = @user
        session[:authenticated_user_id] = @user.id
        flash[:notice] = 'Logged in.'
      else
        flash.now[:warning] = 'Login or password incorrect.'
      end
    end
    
    unless @user
      @user = User.new(params[:user])
    end
    
    if authenticated?
      redirect_to_referer(@authenticated_user)
    end
    
    @referer = params[:referer]
  end
  
  # Session-based logout
  def logout
    session.delete(:authenticated_user_id)
    flash[:notice] = 'Logged out.'
    redirect_to_referer
  end
  
  protected
  def redirect_to_referer(fallback = root_url)
    # Regex checks for protocol in referer to prevent redirection to arbitrary external URIs.
    if params[:referer].instance_of?(String) && (params[:referer] =~ /^\//)
      redirect_to params[:referer]
    else
      redirect_to fallback
    end
  end
end
