# Handles user password retrieval
class PasswordController < ApplicationController
  before_filter :find_key, :only => [ :show, :update ]
    
  # GET /password/new
  # Shows HTML form for new password generation
  def new
    @prk = PasswordResetKey.new
  end
  
  # POST /password
  def create
    unless params[:password_reset_key].blank? || params[:password_reset_key][:user].blank?
      @user_name = params[:password_reset_key][:user][:name]
      @user_email = params[:password_reset_key][:user][:email]
    end

    if !@user_name.blank? && @user_email.blank?
      @user = User.find_by_name(@user_name)
    elsif @user_name.blank? && !@user_email.blank?
      @user = User.find_by_email(@user_email)
    elsif !@user_name.blank? && !@user_email.blank?
      @user = User.find_by_name_and_email(@user_name, @user_email)
    end

    if @user
      if @prk = PasswordResetKey.find_by_user_id(@user.id)
        if @prk.expired?
          # Destroy the expired key
          @prk.destroy
          @prk = nil
        else
          # Use the existing reset key, re-sending the email
          # and extending the expiry (below)
        end
      end
    end

    unless @prk
      @prk = PasswordResetKey.new
      if @user
        @prk.user_id = @user.id
      else
        @user = User.new
      end
    end

    if @prk.save
      # Send email with key to user email address
      begin
        PasswordResetMailer.deliver_key(@user, "noreply@#{ActionMailer::Base.default_url_options[:host]}", @prk)
        respond_to do |format|
          format.xml do 
            http_ok(:text => "Instructions on resetting your password have been sent to the email address associated with your account.")
          end
          format.html
        end
      rescue => e
        logger.error("Mailer error: " + e.inspect)
        @prk.errors.add_to_base("Unable to send email with password reset instructions.")
        respond_to do |format|
          format.xml { http_internal_server_error(@prk.errors) }
          format.html { render :action => 'new' }
        end 
      end
    elsif @prk.errors.on(:user_id) == PasswordResetKey.default_error_messages[:user_id_unique]
      respond_to do |format|
        format.xml { http_conflict(@prk.errors) }
        format.html { render :action => 'new' }
      end
    else
      respond_to do |format|
        format.xml { http_bad_request(@prk.errors) }
        format.html { render :action => 'new' }
      end
    end
  end
  
  # GET /password/{key}
  # Shows an HTML form for user to enter new password
  def show
    respond_to do |format|
      format.html { @user = @prk.user }
    end
  end
  
  # PUT /password/{key}
  def update
    respond_to do |format|
      format.html do
        if params[:password_reset_key] && params[:password_reset_key][:user]
          @prk.user.password = params[:password_reset_key][:user][:password]
          @prk.user.password_confirmation = params[:password_reset_key][:user][:password_confirmation]
          if @prk.user.save
            @prk.destroy
            flash[:notice] = "Password updated. You can now login."
            redirect_to login_url
          else
            @user = @prk.user
            render :action => 'show'
          end
        end
      end
    end
    
#    # Update the user's account with a new, random password
#    new_password = User.random_password
#    @user.password = new_password
#    @user.save
#    
#    begin
#      # Email the new password to the user
#      PasswordResetMailer.deliver_reset(@user, "noreply@#{ActionMailer::Base.default_url_options[:host]}", new_password)
#      http_ok(:text => "Your new password has been sent to the email address associated with your account.")
#      @prk.destroy
#    rescue => e
#      logger.error("Mailer error: " + e.inspect)
#      @prk.errors.add_to_base("Unable to send email with new password.")
#      http_internal_server_error(@prk.errors)
#    end
  end
  
  private
  def find_key
    if @prk = PasswordResetKey.find_by_key(params[:id])
      if @prk.expired?
        @prk.destroy
        flash[:warning] = "This password reset key has expired. Please create another."
        redirect_to new_password_reset_key_url
      end
    else
      http_not_found
    end
  end
end
