class Point < ActiveRecord::Base
  cattr_reader :xml_fields, :default_error_messages
  
  @@xml_fields = [ :description, :name, :position, :uri ]
  
  @@default_error_messages = {
    :uri_format => 'should start with http:// or https://',
    :constant_path_id => 'can not be changed',
    :parent_blank => 'can not be blank and must already exist'
  }
  
  # Prevent mass-assignment of other attributes
  attr_accessible :uri, :name, :description, :position
  
  # Path can not be changed after point creation
  attr_readonly :path_id
    
  # Point is a child of Path
  belongs_to :path
  
  validates_presence_of :uri, :name
  validates_presence_of :path_id, :message => @@default_error_messages[:parent_blank]
  validates_presence_of :position #, :unless => Proc.new { |point| point.new_record? && point.position.nil? }
  validates_uniqueness_of :position, :scope => :path_id #, :unless => Proc.new { |point| point.path_id.blank? || point.path_id_changed? || (point.new_record? && point.position.nil?) }
  validates_numericality_of :position, :only_integer => true, :greater_than => 0, :unless => Proc.new { |point| point.path_id.blank? || point.position.blank? }
  validates_format_of :uri, :with => /^https?:\/\//, :message => @@default_error_messages[:uri_format]
  validates_length_of :uri, :maximum => 2000
  validate_on_update :validate_constant_path_id
  
  before_validation :auto_position!, :if => Proc.new { |point| point.position.blank? }

  def to_param
    self.position_was.to_s
  end
  
  # Set the position of this point to the next available in the path
  def auto_position!
    if self.path ||= Path.find_by_id(self.path_id)
      self.position = self.path.next_position
    end
  end
  
  # Adds an error to path_id if it was changed on a saved point
  def validate_constant_path_id
    if path_id_changed?
      errors.add(:path_id, @@default_error_messages[:constant_path_id])
    end
  end
end
