module PCT
  # Common methods for RESTful resource controllers
  module ResourceController
    CALLBACKS = %w(
      after_save after_create after_update
    )

    def self.included(base) #:nodoc:
      base.send :include, ActiveSupport::Callbacks
      base.define_callbacks *CALLBACKS
    end
        
    # Displays HTML confirmation form for to resource deletion
    def delete
      if request.post? && params[:confirm]
        destroy
      else
        # Only authenticated user can delete their resources
        with_authorization(@user ? @user.name : nil) do
          respond_to do |format|
            format.html
          end
        end
      end
    end
    
    # Displays HTML form for editing resource
    def edit
      # Users can only edit their own resources
      with_authorization(@user ? @user.name : nil) do
        respond_to do |format|
          format.html
        end
      end
    end
    
    private
    def save_resource(resource, url_options, options = {})
      old_url = url_for(url_options)
      new_record = resource.new_record?
      
      # Run controller callbacks
      if saved = resource.save
        run_callbacks :after_save 
        if new_record
          run_callbacks :after_create 
        else
          run_callbacks :after_update
        end
      end
      
      respond_to do |format|
        format.xml do
          if saved
            new_url = url_for(url_options)
            xml = resource.to_xml(:only => options[:xml_only], :include => options[:xml_include])
            if new_record
              http_created(:location => url_options, :xml => xml)
            elsif old_url != new_url
              http_moved_permanently(url_options)
            else
              render :xml => xml
            end
          else
            logger.error("Errors: #{resource.errors.on(:name).inspect}")
            if [resource.errors.on(:name)].flatten.include?(options[:conflict] || I18n.translate('activerecord.errors.messages.taken'))
              http_conflict(resource.errors)
            else
              http_bad_request(resource.errors)
            end
          end
        end
        
        format.html do
          if saved
            flash[:notice] = resource.class.class_name.humanize + (new_record ? " created" : " updated")
            redirect_to url_options
          else
            render :action => new_record ? 'new' : 'edit'
          end
        end
      end
    end
  end
end