ENV["RAILS_ENV"] = "test"
require File.expand_path(File.dirname(__FILE__) + "/../config/environment")
require 'test_help'
require 'rexml/document'

# TODO: Abstract the RESTful interface test logic to DRY up controller functional test scripts
# TODO: Test MethodNotAllowed cases (but the exception is not raised in test environment...)
class ActiveSupport::TestCase
  # Transactional fixtures accelerate your tests by wrapping each test method
  # in a transaction that's rolled back on completion.  This ensures that the
  # test database remains unchanged so your fixtures don't have to be reloaded
  # between every test method.  Fewer database queries means faster tests.
  #
  # Read Mike Clark's excellent walkthrough at
  #   http://clarkware.com/cgi/blosxom/2005/10/24#Rails10FastTesting
  #
  # Every Active Record database supports transactions except MyISAM tables
  # in MySQL.  Turn off transactional fixtures in this case; however, if you
  # don't care one way or the other, switching from MyISAM to InnoDB tables
  # is recommended.
  #
  # The only drawback to using transactional fixtures is when you actually 
  # need to test transactions.  Since your test is bracketed by a transaction,
  # any transactions started in your code will be automatically rolled back.
  self.use_transactional_fixtures = true

  # Instantiated fixtures are slow, but give you @david where otherwise you
  # would need people(:david).  If you don't want to migrate your existing
  # test cases which use the @david style and don't mind the speed hit (each
  # instantiated fixtures translates to a database query per test method),
  # then set this back to true.
  self.use_instantiated_fixtures  = false

  # Setup all fixtures in test/fixtures/*.(yml|csv) for all tests in alphabetical order.
  #
  # Note: You'll currently still have to declare fixtures explicitly in integration tests
  # -- they do not yet inherit this setting
  fixtures :all

  # Custom PCT helper methods available to all tests...
  
  def assert_valid_xml(source)
    doc = nil
    assert_nothing_thrown { doc = REXML::Document.new(source) }
    doc
  end
  
  def assert_valid_xml_response(response, root_element = nil, permitted_children = nil)
    assert_match 'application/xml', response.content_type, 'HTTP response header Content-Type should be application/xml'
    xml = assert_valid_xml(response.body)
    if xml && root_element
      assert_xml_contents(xml, root_element, permitted_children)
    end
    return xml
  end
  
  def assert_xml_contents(doc, root_element, permitted_children = nil)
    # Check that the document root is <path>
    root_xpath = "/#{root_element}"
    assert_equal 1, REXML::XPath.match(doc, root_xpath).size
    
    if permitted_children
      permitted_children = [permitted_children].flatten
      
      # Check that the response contains only expected child elements
      permitted_children.each do |child|
        xpath = "#{root_xpath}/#{child}"
        assert (REXML::XPath.match(doc, xpath).size > 0), "Expected to find element(s) at #{xpath}"
      end
    
      # Check that nothing else is in the root element
      xpath = "#{root_xpath}/*"
      doc.elements.each(xpath) do |element|
        assert permitted_children.include?(element.name), "XML includes unexpected <#{element.name}> element beneath #{root_xpath}"
      end
    end
  end
  
  def assert_response_code(code, message = nil)
    assert_equal code, @response.response_code, build_message(message, "HTTP response code was not as expected")
  end
  
  def http_auth_basic(user_name, password)
	 @request.env['HTTP_AUTHORIZATION'] =  ActionController::HttpAuthentication::Basic.encode_credentials(user_name, password)
	end

  def http_auth_as(user_name)
    case user_name
    when 'bob'
      http_auth_basic('bob', 'secret!')
    when 'admin'
      http_auth_basic('admin', 'admin!')
    when 'jane'
      http_auth_basic('jane', 'johnson!')
    end
  end
    
end
