require 'test_helper'

class CategoryTest < ActiveSupport::TestCase
  # Required attributes
  test "invalid_with_empty_attributes" do
    category = Category.new
    category.valid?
    [:name].each do |column|
      assert_equal I18n.translate('activerecord.errors.messages.blank'), category.errors.on(:name)
    end
  end
  
  # Category name should be unique
  test "unique_name" do
    category = Category.new(:name => categories(:he).name)
    category.valid?
    assert_equal I18n.translate('activerecord.errors.messages.taken'), category.errors.on(:name)
  end
  
  # Valid new category should save
  test "save_valid" do
    category = Category.new(:name => 'New category')
    assert category.save
  end
  
  test "associated paths" do
    assert categories(:general).paths.include?(paths(:empty_path))
    assert !categories(:general).paths.include?(paths(:search_engines_private)), "Private paths should not be included in category listing."
  end
end
